/* classes */

class Board {
  constructor() {
    this.grid = [[0, 1, 2], [3, 4, 5], [6, 7, 8]];
  }
}

class Player {
  constructor() {
    this.mark = null;
  }
}

class Computer {
  constructor() {
    this.mark = null;
  } 
}

/* interaction with user */

function addMarkToPlayer(result) {
  player.mark = result.toString();  
};

function addMarkToComputer(result) {
  computer.mark = result;  
};

function humanMove(move, board) {
  for (let i = 0; i < board.length; i++) {
    for (let j = 0; j < board[i].length; j++) {
      if (board[i][j] === move) {
        board[i][j] = player.mark;
        return;
      };
    }
  }
}

function computerMove(board) {
  for (let i = 0; i < board.length; i++) {
    for (let j = 0; j < board[i].length; j++) {
      if (Number.isInteger(board[i][j])) {
        const computerChoice = (board[i][j]).toString();
        board[i][j] = computer.mark;
        setTimeout(function(){ $("#" + computerChoice).html(computer.mark) }, 450);
        setTimeout(function(){ checkForWinner(board)}, 750);
        return;
      }
    }
  }
}

$(document).ready(function() {
  $('#dialog').dialog({
  	height: "auto",
  	width: 400,
    modal: true,
    buttons: [{
      text: "x",
      click: function() {
        $( this ).dialog( "close" );
        addMarkToPlayer("x");
        addMarkToComputer("o");
      },
      class: "dialog-button"
      }, {
      text: "o", 
      click: function() {
        $( this ).dialog( "close" );
        addMarkToPlayer("o");
        addMarkToComputer("x");
      },
      class: "dialog-button"
    }]
  });

  $(".box").click(function() {
    const humanChoice = parseInt($(this).attr("id"));
    if($(this).html() === "x" || $(this).html() === "o") {
      alert("Whoops! This spot is already taken. Please choose another.");
    } else {
      $(this).html(player.mark); // changes the inner html of the mark
      humanMove(humanChoice, board.grid); // adds the mark to the board
      checkForWinner(board.grid) || computerMove(board.grid);
    }
  });
}); 


/* functions to check the board for a win */

function rowIsFull([head, ...tail]) {
  if (!head && !tail.length) return true; 
  return (Number.isInteger(head)) ? false : rowIsFull(tail);  
}

function checkForTie([head, ...tail]) {
  if (!head && !tail.length) return true;
  return (!rowIsFull(head)) ? false : checkForTie(tail);
};

function rowIsAllSameValue(mark, [head, ...tail]) {
  if (!head && !tail.length) return true; 
  return !(mark === head) ? false : rowIsAllSameValue(mark, tail);  
}

function makeColumns(board) {
  return board.map(function(row, i, board) { 
    return [board[0][i], board[1][i], board[2][i]]
  });
};

function checkRows(mark, board) {
  var head = board[0];
  var tail = board.slice(1, board.length);
  if (!head && !tail.length) return false;
  return  (rowIsAllSameValue(mark, head)) ? true : checkRows(mark, tail);
};

function checkColumns(mark, board) {
  const boardColumns = makeColumns(board);
  return checkRows(mark, boardColumns);
};

function checkLeftDiagonal(mark, board) {
  const leftDiagonal = [board[0][0], board[1][1], board[2][2]];
  return rowIsAllSameValue(mark, leftDiagonal);
}

function checkRightDiagonal(mark, board) {
  const rightDiagonal = [board[0][2], board[1][1], board[2][0]];
  return rowIsAllSameValue(mark, rightDiagonal);
}

function checkXWin(board) {
  if (checkRows("x", board) || 
    checkColumns("x", board) || 
    checkLeftDiagonal("x", board) || 
    checkRightDiagonal("x", board)
  ) {
    console.log("X wins!");
    return true;
  }
};

function checkOWin(board) {
  if (checkRows("o", board) || 
    checkColumns("o", board) || 
    checkLeftDiagonal("o", board) || 
    checkRightDiagonal("o", board)
  ) {
    console.log("O wins!");
    return true;
  }
};

function checkForWinner(board) {
  if (checkXWin(board)) {
    setTimeout(function(){ alert("X wins!"); }, 450);
    setTimeout(function(){ location.reload(); }, 1000);
    return true;
  } else if (checkOWin(board)) {
    setTimeout(function(){ alert("O wins!"); }, 450);
    setTimeout(function(){ location.reload(); }, 1000);
    return true;
  } else if (checkForTie(board)) {
    setTimeout(function(){ alert("It's a tie!"); }, 450);
    setTimeout(function(){ location.reload(); }, 1000);
  }
}

/* game play */

const board = new Board();
const player = new Player();
const computer = new Computer();


